
class NumberPair(tuple[int,int]):
    """Like a tuple, but the order doesn't matter.
    
    It's not like a set, because you can have duplicate items.
    """
    def __eq__(self, value: object) -> bool:
        if isinstance(value, NumberPair):
            return super().__eq__(value) or super().__eq__(NumberPair(reversed(value)))
        return False

    def __hash__(self) -> int:
        (n, m) = self
        if n > m:
            (m, n) = (n, m)
        return (n,m).__hash__()

PotentialPairs = list[int]
DiscoveredPairs = dict[int, set[NumberPair]]

def find_pairs(numbers: PotentialPairs) -> DiscoveredPairs:
    "Discover duplicate sumbs in a list of numbers."
    acc: DiscoveredPairs = {}
    while numbers:
        n = numbers.pop()
        for m in numbers:
            acc[n + m] = acc.get(n + m, set()).union([NumberPair((n, m))])

    return {k: v for (k,v) in acc.items() if len(v) > 1}
